/*
new features + modified + bugs fixed
by Martin Pohlack (mp26@inf.tu-dresden.de)
*/

/*
This source code is placed in the public domain with no guarantee
of fitness for a particular purpose or of portability, but
only with the hope that it may be useful.

I wrote `histogram.c' as a part of my effort to contribute
to a paper by two of my colleagues at SMU. The paper
discussed various programming environments such as Gnuplot,
awk, spice, and xfig.

Mustafa Kocaturk    mustafa@seas.smu.edu  EE Dept., SEAS, SMU
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <iostream.h>

#define HELP \
"This program calculates the histogram of a sequence of numbers\n\
read from the standard input until an end of file is encountered.\n\
The results are printed on the standard output as lines of the format\n\
<subinterval midpoint> <number of sample points in subinterval>\n\
\n\
Usage: %s [number of subintervals (default==100)] [-l <lower bound] [-u <upper bound>]>\n" /* Message string */

#define CALSIZ 2000000 /* Number of doubles to be allocated for sort */

double lower, upper;
int slower = false, supper = false, relative = false;
unsigned int k = 100;

int compare_doubles(const void *p1, const void *p2)
{
    double v1, v2;
    v1 = *((double *)p1);
    v2 = *((double *)p2);

    if (v1 < v2)
	return -1;
    else
	if (v1 > v2)
	    return 1;
	else
	    return 0;
}

int parseOptions(int argc, char *argv[])
{
    int c;

    while ((c = getopt(argc, argv, "l:u:r")) != EOF)
    {
	switch(c)
	{
	case 'l':
	    lower = atof(optarg);
	    slower = true;
	    break;

	case 'u':
	    upper = atof(optarg);
	    supper = true;
	    break;

	case 'r':
	    relative = true;
	    break;

	default:
	    cerr << "Invalid option specified: " << optopt << endl;
	    return false;
	}
    }

    if (argv[optind] != NULL)
    {
	k = atoi(argv[optind]);
	if (argv[optind + 1] != NULL)
	{
	    cerr << "Too many arguments!" << endl << flush;
	    return false;
	}
    }
    return true;
}

int main(int argc, char *argv[])
{
    double x, min, max, lo, hi, step;
    unsigned i, count, num;
    int j = 1;
    char c;

    double input[CALSIZ];

    if (!parseOptions(argc, argv))
    {
	fprintf(stderr,HELP,argv[0]);
	exit(1);
    }

    // read number, skip other lines, break if eof or CALSIZ read
    for (i = 0; i < CALSIZ && j > 0;)
    {
	if((j = scanf("%lf%*[^\n]%*c",&x))>0)  // single number
	{
	    input[i] = x;
	    i++;
	    continue;
	}
	else
	    j=scanf("%*[^\n]%*c");
	// Read data until end-of-line, if end-of-stdin (i. e. ctrl-D) then j<0 (i. e. break the for loop)
    }

    if (i >= CALSIZ)
	cerr << "Warning: buffer limit reached, skipped rest of data!" << endl << flush;

    count = i;

    // now sort input
    qsort(input, count, sizeof(double), compare_doubles);
    
    //for (i = 0; i < count ; i++)
      //	cerr << input[i] << ", ";

    if (!slower)
	min = input[0];
    else
	min = lower;

    if (!supper)
	max = input[count - 1];
    else
	max = upper;

    step = (max - min) / k;

    cerr << "Using bins = " << k << ", l = " << min << ", u = " << max << ", step = " << step << endl << flush;

    lo = min;
    hi = min + step;
    num = 0;

    // step to first data in range
    for (i = 0; input[i] < min; i++);

    for (j = 0; i < count && j < k && input[i] <= max;)
    {
	// no, this interval is intentionally [] and not (] or [), but that is not an error!
	// think carefully ...
	if (input[i] >= lo && input[i] <= hi)  // still previous interval?
	{
	    num++;
	    i++;
	}
	else
	{
	    if (relative) // decide whether absolute or relative probabilities
		printf("%g %g\n", lo + step * 0.5, (double)((double)num / (double)count)); // Print subinterval center and number of values in it
	    else
		printf("%g %d\n", lo + step * 0.5, num); // Print subinterval center and number of values in it
	    j++;                                     // next interval
	    lo = hi;                                 // update borders
	    hi = max - (k - j - 1) * step;
	    num = 0;                                 // and reset count
	}
    }
    if (relative) // decide whether absolute or relative probabilities
 	printf("%g %g\n", lo + step * 0.5, (double)((double)num / (double)count)); // the last one
    else
 	printf("%g %d\n", lo + step * 0.5, num); // the last one
    exit(0);
}
