#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <iostream.h>
#include <iomanip.h>

#define HELP \
"This program calculates the histogram of a sequence of numbers\n\
read from the standard input until an end of file is encountered.\n\
The results are printed on the standard output as lines of the format\n\
<bin midpoint> <number of sample points in subinterval> (as gnuplot likes it)\n\
\n\
Usage: %s -x <low bound> -X <high bound> -w <# of bins>\n\
Example: %s -x -5.0 -X 5.0 -w 10 < in.dat > out.dat\n" /* Message string */

double xl = 0.0, xh = 1.0, xs;
int xw = 4;

int parseOptions(int argc, char *argv[])
{
	int c;

	while ((c = getopt(argc, argv, "x:X:w:")) != EOF)
	{
		switch(c)
		{
		case 'x':
			xl = atof(optarg);
			break;
		case 'X':
			xh = atof(optarg);
			break;
		case 'w':
			xw = atoi(optarg);
			break;

		default:
			cerr << "Invalid option specified: " << optopt << endl;
			return false;
		}
    }

	if (argv[optind] != NULL)
	{
		cerr << "Too many arguments!" << endl << flush;
		return false;
	}

	if ((xl >= xh) || (xw < 1))
	{
		cerr << "Wrong arguments!" << endl << flush;
		return false;
	}

	return true;
}

int main(int argc, char *argv[])
{
	int count = 0;
	double x;

	// get args
	if (!parseOptions(argc, argv))
	{
		fprintf(stderr,HELP,argv[0], argv[0]);
		exit(1);
	}

	// compute bin-borders etc.
	xs = (xh - xl) / xw;

	cerr << "Interval: (" << xl << ", " << xh << "), "
	     << "bin size=" << xs << ", bin count=" << xw << endl;

	int field[xw];// = new int[xw];

	for (int x = 0; x < xw; x++)
		field[x] = 0;

	while (true)
	{
		/* doesn't work for some reason
		// last line get read twice ->?
		// so using the old way
		cin >> x >> y;
		*/
		int j;
		j = scanf("%lf%*[^\n]%*c",&x);
		if (j < 0)
			break;  // eof
		if (j != 1)
		{
			cerr << "Unknown format found, stopping import!" << endl;
			break;
		}

		if ((x >= xl) && (x < xh))
		{
			field[(int)((x - xl) / xs)]++;
			count++;
		}
	}

	cout << setw(10) << setprecision(10);
	for (int x = 0; x < xw; x++)
	{
		cout << xl + ((x + 0.5) * (xh - xl)) / (double)xw << "\t"
		     << field[x] << endl;
	}
	//cerr << count << endl;
}
