#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <iostream.h>
#include <iomanip.h>

#define HELP \
"This program calculates the histogram of a sequence of number pairs\n\
read from the standard input until an end of file is encountered.\n\
The results are printed on the standard output as lines of the format\n\
<bin midpoint x> <bin midpoint y> <number of sample points in subinterval>\n\
(as gnuplot likes it)\n\
\n\
Usage: %s -x <low bound> -X <high bound> -y <low bound> -y <high bound> -w <# of bins in x dim> -d <# of bins in y dim>\n\
Example: %s -x -5.0 -X 5.0 -y -2.0 -Y 7.0 -w 10 -d 9 < in.dat > out.dat\n" /* Message string */

double xl = 0.0, xh = 1.0, yl = 0.0, yh = 1.0, xs, ys;
int xw = 4, yw = 4;

int parseOptions(int argc, char *argv[])
{
	int c;

	while ((c = getopt(argc, argv, "x:y:X:Y:w:d:")) != EOF)
	{
		switch(c)
		{
		case 'x':
			xl = atof(optarg);
			break;
		case 'y':
			yl = atof(optarg);
			break;
		case 'X':
			xh = atof(optarg);
			break;
		case 'Y':
			yh = atof(optarg);
			break;
		case 'w':
			xw = atoi(optarg);
			break;
		case 'd':
			yw = atoi(optarg);
			break;

		default:
			cerr << "Invalid option specified: " << optopt << endl;
			return false;
		}
	}

	if (argv[optind] != NULL)
	{
		cerr << "Too many arguments!" << endl << flush;
		return false;
	}

	if ((xl >= xh) || (yl >= yh) || (xw < 1) || (yw < 1))
	{
		cerr << "Wrong arguments!" << endl << flush;
		return false;
	}

	return true;
}

int main(int argc, char *argv[])
{
	int count = 0;
	double x, y;

	// get args
	if (!parseOptions(argc, argv))
	{
		fprintf(stderr,HELP,argv[0], argv[0]);
		exit(1);
	}

	cerr << "Using: (" << xl << ", " << xh << ")"
	     << "(" << yl << ", " << yh << ")" << "X=" << xw << ", Y=" << yw << endl;

	// compute bin-borders etc.
	xs = (xh - xl) / xw;
	ys = (yh - yl) / yw;

	int field[xw][yw];// = new int[xw][yw];

	for (int x = 0; x < xw; x++)
		for (int y = 0; y < yw; y++)
			field[x][y] = 0;

	while (true)
	{
		/* doesn't work for some reason
		// last line get read twice ->?
		// so using the old way
		cin >> x >> y;
		*/
		int j;
		j = scanf("%lf%*[ \t]%lf%*[^\n]%*c",&x, &y);
		if (j < 0)
			break;  // eof
		if (j != 2)
		{
			cerr << "Unknown format found, stopping import!" << endl;
			break;
		}

		if ((x >= xl) && (x < xh) && (y >= yl) && (y < yh))
		{
			//cerr << x << ", " << y << endl;
			//cerr << ((x - xl) / xs) << ", " << ((y - yl) / ys) << endl;
			field[(int)((x - xl) / xs)][(int)((y - yl) / ys)]++;
			count++;
		}
	}

	cout << setw(10) << setprecision(10);
	for (int x = 0; x < xw; x++)
	{
		for (int y = 0; y < yw; y++)
		{
			cout << xl + ((x + 0.5) * (xh - xl)) / (double)xw << "\t"
			<< yl + ((y + 0.5) * (yh - yl)) / (double)yw << "\t"
			<< field[x][y] << endl;
		}
		cout << endl;
	}
	//cerr << count << endl;
}
